package javax.swing.plaf.synth;

import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.awt.Insets;
import java.awt.Container;
import java.awt.Dimension;
import javax.swing.plaf.SpinnerUI;
import java.beans.PropertyChangeEvent;
import java.awt.Graphics;
import java.awt.Component;
import java.awt.LayoutManager;
import javax.swing.plaf.UIResource;
import javax.swing.JFormattedTextField;
import javax.swing.JSpinner;
import javax.swing.plaf.ComponentUI;
import javax.swing.JComponent;
import java.beans.PropertyChangeListener;
import javax.swing.plaf.basic.BasicSpinnerUI;

public class SynthSpinnerUI extends BasicSpinnerUI implements PropertyChangeListener, SynthUI
{
    private SynthStyle style;
    private SynthSpinnerUI.EditorFocusHandler editorFocusHandler;
    
    public SynthSpinnerUI() {
        this.editorFocusHandler = new SynthSpinnerUI.EditorFocusHandler();
    }
    
    public static ComponentUI createUI(JComponent component) {
        return new SynthSpinnerUI();
    }
    
    @Override
    protected void installListeners() {
        super.installListeners();
        this.spinner.addPropertyChangeListener(this);
        JComponent editor = this.spinner.getEditor();
        if ((editor instanceof JSpinner.DefaultEditor)) {
            JFormattedTextField textField = ((JSpinner.DefaultEditor)editor).getTextField();
            if (textField != null) {
                textField.addFocusListener(this.editorFocusHandler);
            }
        }
    }
    
    @Override
    protected void uninstallListeners() {
        super.uninstallListeners();
        this.spinner.removePropertyChangeListener(this);
        JComponent editor = this.spinner.getEditor();
        if ((editor instanceof JSpinner.DefaultEditor)) {
            JFormattedTextField textField = ((JSpinner.DefaultEditor)editor).getTextField();
            if (textField != null) {
                textField.removeFocusListener(this.editorFocusHandler);
            }
        }
    }
    
    @Override
    protected void installDefaults() {
        LayoutManager layout = this.spinner.getLayout();
        if (layout == null || layout instanceof UIResource) {
            this.spinner.setLayout(this.createLayout());
        }
        this.updateStyle(this.spinner);
    }
    
    private void updateStyle(JSpinner spinner) {
        SynthContext context = this.getContext(spinner, 1);
        SynthStyle style = this.style;
        this.style = SynthLookAndFeel.updateStyle(context, this);
        if (this.style != style && style != null) {
            this.installKeyboardActions();
        }
        context.dispose();
    }
    
    @Override
    protected void uninstallDefaults() {
        if ((this.spinner.getLayout() instanceof UIResource)) {
            this.spinner.setLayout(null);
        }
        SynthContext context = this.getContext(this.spinner, 1);
        this.style.uninstallDefaults(context);
        context.dispose();
        this.style = null;
    }
    
    @Override
    protected LayoutManager createLayout() {
        return new SynthSpinnerUI.SpinnerLayout();
    }
    
    @Override
    protected Component createPreviousButton() {
        SynthArrowButton c = new SynthArrowButton(5);
        c.setName("Spinner.previousButton");
        this.installPreviousButtonListeners(c);
        return c;
    }
    
    @Override
    protected Component createNextButton() {
        SynthArrowButton c = new SynthArrowButton(1);
        c.setName("Spinner.nextButton");
        this.installNextButtonListeners(c);
        return c;
    }
    
    @Override
    protected JComponent createEditor() {
        JComponent editor = this.spinner.getEditor();
        editor.setName("Spinner.editor");
        this.updateEditorAlignment(editor);
        return editor;
    }
    
    @Override
    protected void replaceEditor(JComponent comp, JComponent comp2) {
        this.spinner.remove(comp);
        this.spinner.add(comp2, "Editor");
        if ((comp instanceof JSpinner.DefaultEditor)) {
            JFormattedTextField textField = ((JSpinner.DefaultEditor)comp).getTextField();
            if (textField != null) {
                textField.removeFocusListener(this.editorFocusHandler);
            }
        }
        if ((comp2 instanceof JSpinner.DefaultEditor)) {
            JFormattedTextField textField2 = ((JSpinner.DefaultEditor)comp2).getTextField();
            if (textField2 != null) {
                textField2.addFocusListener(this.editorFocusHandler);
            }
        }
    }
    
    private void updateEditorAlignment(JComponent component) {
        if ((component instanceof JSpinner.DefaultEditor)) {
            SynthContext context = this.getContext(this.spinner);
            Integer n = (Integer)context.getStyle().get(context, "Spinner.editorAlignment");
            JFormattedTextField textField = ((JSpinner.DefaultEditor)component).getTextField();
            if (n != null) {
                textField.setHorizontalAlignment((int)n);
            }
            textField.putClientProperty("JComponent.sizeVariant", this.spinner.getClientProperty("JComponent.sizeVariant"));
        }
    }
    
    @Override
    public SynthContext getContext(JComponent component) {
        return this.getContext(component, SynthLookAndFeel.getComponentState(component));
    }
    
    private SynthContext getContext(JComponent component, int n) {
        return SynthContext.getContext(component, this.style, n);
    }
    
    @Override
    public void update(Graphics graphics, JComponent component) {
        SynthContext context = this.getContext(component);
        SynthLookAndFeel.update(context, graphics);
        context.getPainter().paintSpinnerBackground(context, graphics, 0, 0, component.getWidth(), component.getHeight());
        this.paint(context, graphics);
        context.dispose();
    }
    
    @Override
    public void paint(Graphics graphics, JComponent component) {
        SynthContext context = this.getContext(component);
        this.paint(context, graphics);
        context.dispose();
    }
    
    protected void paint(SynthContext synthContext, Graphics graphics) {
    }
    
    @Override
    public void paintBorder(SynthContext synthContext, Graphics graphics, int n, int n2, int n3, int n4) {
        synthContext.getPainter().paintSpinnerBorder(synthContext, graphics, n, n2, n3, n4);
    }
    
    @Override
    public void propertyChange(PropertyChangeEvent propertyChangeEvent) {
        JSpinner spinner = (JSpinner)(JSpinner)propertyChangeEvent.getSource();
        SpinnerUI ui = spinner.getUI();
        if ((ui instanceof SynthSpinnerUI)) {
            SynthSpinnerUI synthSpinnerUI = (SynthSpinnerUI)ui;
            if (SynthLookAndFeel.shouldUpdateStyle(propertyChangeEvent)) {
                synthSpinnerUI.updateStyle(spinner);
            }
        }
    }
    
    private static class SpinnerLayout implements LayoutManager, UIResource
    {
        private Component nextButton;
        private Component previousButton;
        private Component editor;
        
        private SpinnerLayout() {
            this.nextButton = null;
            this.previousButton = null;
            this.editor = null;
        }
        
        @Override
        public void addLayoutComponent(String anObject, Component nextButton) {
            if ("Next".equals(anObject)) {
                this.nextButton = nextButton;
            }
            else if ("Previous".equals(anObject)) {
                this.previousButton = nextButton;
            }
            else if ("Editor".equals(anObject)) {
                this.editor = nextButton;
            }
        }
        
        @Override
        public void removeLayoutComponent(Component component) {
            if (component == this.nextButton) {
                this.nextButton = null;
            }
            else if (component == this.previousButton) {
                this.previousButton = null;
            }
            else if (component == this.editor) {
                this.editor = null;
            }
        }
        
        private Dimension preferredSize(Component component) {
            return (component == null) ? new Dimension(0, 0) : component.getPreferredSize();
        }
        
        @Override
        public Dimension preferredLayoutSize(Container container) {
            Dimension preferredSize = this.preferredSize(this.nextButton);
            Dimension preferredSize2 = this.preferredSize(this.previousButton);
            Dimension preferredSize3 = this.preferredSize(this.editor);
            preferredSize3.height = (preferredSize3.height + 1) / 2 * 2;
            Dimension dimension = new Dimension(preferredSize3.width, preferredSize3.height);
            dimension.width += Math.max(preferredSize.width, preferredSize2.width);
            Insets insets = container.getInsets();
            dimension.width += insets.left + insets.right;
            dimension.height += insets.top + insets.bottom;
            return dimension;
        }
        
        @Override
        public Dimension minimumLayoutSize(Container container) {
            return this.preferredLayoutSize(container);
        }
        
        private void setBounds(Component component, int x, int y, int width, int height) {
            if (component != null) {
                component.setBounds(x, y, width, height);
            }
        }
        
        @Override
        public void layoutContainer(Container container) {
            Insets insets = container.getInsets();
            int n = container.getWidth() - (insets.left + insets.right);
            int n2 = container.getHeight() - (insets.top + insets.bottom);
            Dimension preferredSize = this.preferredSize(this.nextButton);
            Dimension preferredSize2 = this.preferredSize(this.previousButton);
            int n3 = n2 / 2;
            int n4 = n2 - n3;
            int max = Math.max(preferredSize.width, preferredSize2.width);
            int n5 = n - max;
            int left;
            int left2;
            if (container.getComponentOrientation().isLeftToRight()) {
                left2 = insets.left;
                left = left2 + n5;
            }
            else {
                left = insets.left;
                left2 = left + max;
            }
            int n6 = insets.top + n3;
            this.setBounds(this.editor, left2, insets.top, n5, n2);
            this.setBounds(this.nextButton, left, insets.top, max, n3);
            this.setBounds(this.previousButton, left, n6, max, n4);
        }
    }
    
    private class EditorFocusHandler implements FocusListener
    {
        @Override
        public void focusGained(FocusEvent focusEvent) {
            SynthSpinnerUI.this.spinner.repaint();
        }
        
        @Override
        public void focusLost(FocusEvent focusEvent) {
            SynthSpinnerUI.this.spinner.repaint();
        }
    }
}
